@@ -10,19 +10,14 @@
 #include <mutex>
 #include <chrono>
 
+#include "larcv3/core/dataformat/Particle.h"
+
 #ifdef LARCV_OPENMP
 #include <omp.h>
 #endif
 
 namespace larcv3 {
-#ifdef LARCV_OPENMP
-int omp_thread_count() {
-    int n = 0;
-    #pragma omp parallel reduction(+:n)
-    n += 1;
-    return n;
-}
-#endif
+
 
 
   QueueProcessor::QueueProcessor(std::string name)
@@ -41,19 +36,18 @@ int omp_thread_count() {
 
   const std::string& QueueProcessor::storage_name(size_t process_id) const
   {
-    if (process_id > _process_name_v.size()) {
-      LARCV_CRITICAL() << "Process ID " << process_id << " is invalid!" << std::endl;
-      throw larbys();
-    }
-    return _process_name_v[process_id];
+
+    return _driver.process_ptr(process_id)->name();
+
   }
 
   size_t QueueProcessor::process_id(const std::string& name) const
   {
-    for (size_t id = 0; id < _process_name_v.size(); ++id)
-      if (name == _process_name_v[id]) return id;
-    LARCV_ERROR() << "Could not locate process name: " << name << std::endl;
-    return kINVALID_SIZE;
+    return _driver.process_id(name);
+    // for (size_t id = 0; id < _process_name_v.size(); ++id)
+    //   if (name == _process_name_v[id]) return id;
+    // LARCV_ERROR() << "Could not locate process name: " << name << std::endl;
+    // return kINVALID_SIZE;
   }
 
 
@@ -97,11 +91,54 @@ int omp_thread_count() {
 
     _next_index_v.resize(buf.size);
     size_t * ptr = (size_t *) buf.ptr;
-    for (size_t i = 0; i < buf.size; i++ ){
+    for (pybind11::ssize_t i = 0; i < buf.size; i++ ){
       _next_index_v[i] = ptr[i];
     }
   }
 
+  template<class T>
+  const BatchDataQueue<T> & QueueProcessor::get_queue(std::string process_name) const{
+      return BatchDataQueueFactory<T>::get().get_queue(process_name);
+  }
+
+  bool QueueProcessor::is_next_ready() const {
+
+      bool ready = true;
+
+      // Next, Check on all the factories:
+      for (auto const& process_name : _driver.process_names()) {
+
+        // Get the process ID by name lookup
+        ProcessID_t pid = _driver.process_id(process_name);
+        auto proc_ptr = _driver.process_ptr(pid);
+
+        if (!(proc_ptr->is("BatchFiller"))) continue;
+        auto datatype = ((BatchHolder*)(proc_ptr))->data_type() ;
+
+
+        switch ( datatype ) {
+        case BatchDataType_t::kBatchDataShort:
+          ready = ready && BatchDataQueueFactory<short>::get().get_queue(process_name).is_next_ready(); break;
+        case BatchDataType_t::kBatchDataInt:
+          ready = ready && BatchDataQueueFactory<int>::get().get_queue(process_name).is_next_ready(); break;
+        case BatchDataType_t::kBatchDataFloat:
+          ready = ready && BatchDataQueueFactory<float>::get().get_queue(process_name).is_next_ready(); break;
+        case BatchDataType_t::kBatchDataDouble:
+          ready = ready && BatchDataQueueFactory<double>::get().get_queue(process_name).is_next_ready(); break;
+        case BatchDataType_t::kBatchDataParticle:
+          ready = ready && BatchDataQueueFactory<larcv3::ParticleHolder>::get().get_queue(process_name).is_next_ready(); break;
+        default:
+          LARCV_CRITICAL() << "Process name " << process_name
+                           << " encountered none-supported BatchDataType_t: " << int(datatype) << std::endl;
+          throw larbys();
+        }
+      }
+      return ready;
+  }
+
+  bool QueueProcessor::is_reading() const {
+    return !(_preparation_future.wait_for(std::chrono::seconds(0)) == std::future_status::ready);
+  }
 
   void QueueProcessor::reset()
   {
@@ -124,84 +161,40 @@ int omp_thread_count() {
     _batch_global_counter = 0;
   }
 
-  void QueueProcessor::configure(const std::string config_file, int color)
-  {
-    LARCV_DEBUG() << "Called" << std::endl;
-    // check state
-    if (_processing) {
-      LARCV_CRITICAL() << "Must call finalize() before calling initialize() after starting to process..." << std::endl;
-      throw larbys();
-    }
-    // check cfg file
-    if (config_file.empty()) {
-      LARCV_CRITICAL() << "Config file not set!" << std::endl;
-      throw larbys();
-    }
-
-    // check cfg content top level
-    auto main_cfg = CreatePSetFromFile(config_file);
-    if (!main_cfg.contains_pset(name())) {
-      LARCV_CRITICAL() << "QueueProcessor configuration (" << name() << ") not found in the config file (dump below)" << std::endl
-                       << main_cfg.dump()
-                       << std::endl;
-      throw larbys();
-    }
-    auto const cfg = main_cfg.get<larcv3::PSet>(name());
-    configure(cfg);
-  }
-
   void QueueProcessor::pop_current_data()
   {
 
-    bool ready = true;
 
-    for (size_t pid = 0; pid < _process_name_v.size(); ++pid) {
-      auto const& name = _process_name_v[pid];
-
-      auto proc_ptr = _driver.process_ptr(pid);
-      if (!(proc_ptr->is("BatchFiller"))) continue;
-      auto datatype = ((BatchHolder*)(proc_ptr))->data_type() ;
-
-
-      switch ( datatype ) {
-      case BatchDataType_t::kBatchDataShort:
-        ready = ready && BatchDataQueueFactory<short>::get().get_queue(name).is_next_ready(); break;
-      case BatchDataType_t::kBatchDataInt:
-        ready = ready && BatchDataQueueFactory<int>::get().get_queue(name).is_next_ready(); break;
-      case BatchDataType_t::kBatchDataFloat:
-        ready = ready && BatchDataQueueFactory<float>::get().get_queue(name).is_next_ready(); break;
-      case BatchDataType_t::kBatchDataDouble:
-        ready = ready && BatchDataQueueFactory<double>::get().get_queue(name).is_next_ready(); break;
-      default:
-        LARCV_CRITICAL() << "Process name " << name
-                         << " encountered none-supported BatchDataType_t: " << int(datatype) << std::endl;
-        throw larbys();
-      }
-    }
+    bool ready = this->is_next_ready();
 
     if (! ready){
       LARCV_ERROR() << "Can't pop current data because the next data is not yet ready." << std::endl;
     }
 
-    for (size_t pid = 0; pid < _process_name_v.size(); ++pid) {
-      auto const& name = _process_name_v[pid];
+    for (auto const& process_name : _driver.process_names()) {
 
+      // Get the process ID by name lookup
+      ProcessID_t pid = _driver.process_id(process_name);
       auto proc_ptr = _driver.process_ptr(pid);
+
+
       if (!(proc_ptr->is("BatchFiller"))) continue;
-      // _batch_filler_id_v.push_back(pid);
+
       auto datatype = ((BatchHolder*)(proc_ptr))->data_type() ;
 
       switch ( datatype ) {
       case BatchDataType_t::kBatchDataShort:
-        BatchDataQueueFactory<short>::get_writeable().get_queue_writeable(name).pop(); break;
+        BatchDataQueueFactory<short>::get_writeable().get_queue_writeable(process_name).pop(); break;
       case BatchDataType_t::kBatchDataInt:
-        BatchDataQueueFactory<int>::get_writeable().get_queue_writeable(name).pop(); break;
+        BatchDataQueueFactory<int>::get_writeable().get_queue_writeable(process_name).pop(); break;
       case BatchDataType_t::kBatchDataFloat:
-        BatchDataQueueFactory<float>::get_writeable().get_queue_writeable(name).pop(); break;
+        BatchDataQueueFactory<float>::get_writeable().get_queue_writeable(process_name).pop(); break;
       case BatchDataType_t::kBatchDataDouble:
-        BatchDataQueueFactory<double>::get_writeable().get_queue_writeable(name).pop(); break;
+        BatchDataQueueFactory<double>::get_writeable().get_queue_writeable(process_name).pop(); break;
+      case BatchDataType_t::kBatchDataParticle:
+        BatchDataQueueFactory<larcv3::ParticleHolder>::get_writeable().get_queue_writeable(process_name).pop(); break;
       default:
-        LARCV_CRITICAL() << "Process name " << name
+        LARCV_CRITICAL() << "Process name " << process_name
                          << " encountered none-supported BatchDataType_t: " << int(datatype) << std::endl;
         throw larbys();
       }
@@ -213,120 +206,102 @@ int omp_thread_count() {
     _current_batch_entries_v = std::move(_next_batch_entries_v);
   }
 
-  void QueueProcessor::configure(const PSet& orig_cfg, int color)
+  void QueueProcessor::configure(const json& orig_cfg, int color)
   {
     reset();
+    config = this->default_config();
+    larcv3::logger::default_level(msg::kDEBUG);
+
+    // Get the default config
+
+    // Add the user arguments
+    config = augment_default_config(config, orig_cfg);
 
 
-    // std::cout << "\033[93m setting verbosity \033[00m" << orig_cfg.get<unsigned short>("Verbosity", 2) << std::endl;
-    set_verbosity( (msg::Level_t)(orig_cfg.get<unsigned short>("Verbosity", 2)) );
+    set_verbosity( config["Verbosity"].get<msg::Level_t>() );
+
+    auto _input_fname_v = config["InputFiles"].get<std::vector<std::string>>();
 
-    _input_fname_v = orig_cfg.get<std::vector<std::string> >("InputFiles");
 
     _process_name_v.clear();
 
+
     // Initialize the ProcessDriver:
 
+    // To init the process driver, we pull the default config (which includes an IOConfig)
+    // and then update as needed.
 
-    std::stringstream ss_tmp1;
-    ss_tmp1 << name();
+    // Three important pieces here to fill in:
 
-    std::string proc_name(ss_tmp1.str());
-    std::string io_cfg_name = "IOManager";
+    // ProcessType
+    // ProcessName
+    // ProcessList
 
-    LARCV_INFO() << "Constructing Processor config: " << proc_name << std::endl;
-    PSet proc_cfg(proc_name);
-    for (auto const& value_key : orig_cfg.value_keys()) {
-      if (value_key == "ProcessName") {
-        for (auto const& unit_name : orig_cfg.get<std::vector<std::string> >("ProcessName")) {
-          _process_name_v.push_back(unit_name);
-        }
-      }
-      if (value_key == "RandomAccess"){
-        continue;
-      }
-      proc_cfg.add_value(value_key, orig_cfg.get<std::string>(value_key));
+    // We have already the default config, so it's a matter of making sure we've got the rest.
+
+
+
+
+    auto & proc_cfg = config["ProcessDriver"];
+
+
+    LARCV_INFO() << "Enforcing configuration ..." << std::endl;
+
+
+    // configure the driver
+    _driver.reset();
+    _driver.configure(proc_cfg);
+    _driver.override_input_file(_input_fname_v);
+
+
+    LARCV_NORMAL() << "Done configuration ..." << std::endl;
+
+    // Check & report batch filler's presence
+    for (auto const& process_name : _driver.process_names()) {
+      ProcessID_t id = _driver.process_id(process_name);
+
+      auto ptr = _driver.process_ptr(id);
+
+      LARCV_INFO() << "Process " << process_name << " ... ID=" << id << "... BatchFiller? : " << ptr->is("BatchFiller") << std::endl;
+      _process_name_v.push_back(process_name);
     }
-    proc_cfg.add_value("RandomAccess", "false");
-
-    // Brew read-only configuration
-    PSet io_cfg(io_cfg_name);
-    io_cfg.add_value("Verbosity", std::string(std::to_string(logger().level())));
-    io_cfg.add_value("Name", io_cfg_name);
-    io_cfg.add_value("IOMode", "0");
-    io_cfg.add_value("OutFileName", "");
-    io_cfg.add_value("StoreOnlyType", "[]");
-    io_cfg.add_value("StoreOnlyName", "[]");
-    // io_cfg.add_value("UseH5CoreDriver", "true");
+    _driver.initialize(color);
 
 
-    LARCV_INFO() << "Constructing IO configuration: " << io_cfg_name << std::endl;
+    // only-once-operation among all queueus: initialize storage
+    _batch_filler_id_v.clear();
+    _batch_data_type_v.clear();
+    for (auto const& process_name : _driver.process_names()) {
 
-    for (auto const& pset_key : orig_cfg.pset_keys()) {
-      if (pset_key == "IOManager") {
-        // auto const& orig_io_cfg = orig_cfg.get_pset(pset_key);
-        LARCV_NORMAL() << "IOManager configuration will be ignored..." << std::endl;
-      }
-      else if (pset_key == "ProcessList") {
-        auto const& orig_thread_plist = orig_cfg.get<larcv3::PSet>(pset_key);
-        proc_cfg.add_pset(orig_thread_plist);
-      }
-      else
-        proc_cfg.add_pset(orig_cfg.get_pset(pset_key));
+      // Get the process ID by name lookup
+      ProcessID_t id = _driver.process_id(process_name);
+      auto proc_ptr = _driver.process_ptr(id);
 
+      // Skip non batch fillers
+      if (!(proc_ptr->is("BatchFiller"))) continue;
 
-
-      proc_cfg.add_pset(io_cfg);
-
-
-      LARCV_INFO() << "Enforcing configuration ..." << std::endl;
-
-      LARCV_INFO() << proc_cfg.dump() << std::endl;
-
-      // configure the driver
-      _driver.reset();
-      _driver.configure(proc_cfg);
-      _driver.override_input_file(_input_fname_v);
-
-
-      LARCV_NORMAL() << "Done configuration ..." << std::endl;
-
-      // Check & report batch filler's presence
-      for (auto const& process_name : _driver.process_names()) {
-        ProcessID_t id = _driver.process_id(process_name);
-
-        auto ptr = _driver.process_ptr(id);
-
-        LARCV_INFO() << "Process " << process_name << " ... ID=" << id << "... BatchFiller? : " << ptr->is("BatchFiller") << std::endl;
-      }
-      _driver.initialize(color);
-
-      // only-once-operation among all queueus: initialize storage
-      _batch_filler_id_v.clear();
-      _batch_data_type_v.clear();
-      for (size_t pid = 0; pid < _process_name_v.size(); ++pid) {
-        auto proc_ptr = _driver.process_ptr(pid);
-        if (!(proc_ptr->is("BatchFiller"))) continue;
-        _batch_filler_id_v.push_back(pid);
-        auto datatype = ((BatchHolder*)(proc_ptr))->data_type() ;
-        _batch_data_type_v.push_back(datatype);
-        auto const& name = _process_name_v[pid];
-        switch ( datatype ) {
-        case BatchDataType_t::kBatchDataShort:
-          BatchDataQueueFactory<short>::get_writeable().make_queue(name); break;
-        case BatchDataType_t::kBatchDataInt:
-          BatchDataQueueFactory<int>::get_writeable().make_queue(name); break;
-        case BatchDataType_t::kBatchDataFloat:
-          BatchDataQueueFactory<float>::get_writeable().make_queue(name); break;
-        case BatchDataType_t::kBatchDataDouble:
-          BatchDataQueueFactory<double>::get_writeable().make_queue(name); break;
-        default:
-          LARCV_CRITICAL() << "Process name " << name
-                           << " encountered none-supported BatchDataType_t: " << (int)(((BatchHolder*)(proc_ptr))->data_type()) << std::endl;
-          throw larbys();
-        }
+      // Save the id and datatype
+      auto datatype = ((BatchHolder*)(proc_ptr))->data_type() ;
+      _batch_filler_id_v.push_back(id);
+      _batch_data_type_v.push_back(datatype);
+      switch ( datatype ) {
+      case BatchDataType_t::kBatchDataShort:
+        BatchDataQueueFactory<short>::get_writeable().make_queue(process_name); break;
+      case BatchDataType_t::kBatchDataInt:
+        BatchDataQueueFactory<int>::get_writeable().make_queue(process_name); break;
+      case BatchDataType_t::kBatchDataFloat:
+        BatchDataQueueFactory<float>::get_writeable().make_queue(process_name); break;
+      case BatchDataType_t::kBatchDataDouble:
+        BatchDataQueueFactory<double>::get_writeable().make_queue(process_name); break;
+      case BatchDataType_t::kBatchDataParticle:
+        BatchDataQueueFactory<larcv3::ParticleHolder>::get_writeable().make_queue(process_name); break;
+      default:
+        LARCV_CRITICAL() << "Process name " << process_name
+                         << " encountered none-supported BatchDataType_t: " << (int)(((BatchHolder*)(proc_ptr))->data_type()) << std::endl;
+        throw larbys();
       }
     }
+
     _configured = true;
   }
 
@@ -336,6 +311,7 @@ int omp_thread_count() {
     _preparation_future.wait();
 
 
+
     _processing = true;
     std::future<bool> fut = std::async(std::launch::async,
                                        &QueueProcessor::batch_process, this);
@@ -379,19 +355,9 @@ int omp_thread_count() {
     LARCV_INFO() << "Entering process loop" << std::endl;
     // auto start = std::chrono::steady_clock::now();
 
-#ifdef LARCV_OPENMP
-    std::cout << "Number of threads: " << omp_thread_count() << std::endl;
-#endif
-    // size_t i(0),
     size_t i_entry(0);
 
-    // #pragma omp parallel
-    // {
-    //   #pragma omp single
-    //   {
         for(i_entry =0; i_entry < _next_index_v.size(); ++ i_entry){
-          // #pragma omp task
-          // {
             auto & entry = _next_index_v[i_entry];
             LARCV_INFO() << "Processing entry: " << entry << std::endl;
 
@@ -400,11 +366,7 @@ int omp_thread_count() {
             LARCV_INFO() << "Finished processing event id: " << _driver.event_id().event_key() << std::endl;
             _next_batch_entries_v.at(i_entry) = entry;
             _next_batch_events_v.at(i_entry) = _driver.event_id();
-          // }
         }
-    //   }
-    // }
-
 
     // auto duration = std::chrono::duration_cast< std::chrono::milliseconds>(std::chrono::steady_clock::now() - start);
 
@@ -425,35 +387,44 @@ int omp_thread_count() {
 
   bool QueueProcessor::set_batch_storage(){
 
-    for (size_t pid = 0; pid < _process_name_v.size(); ++pid) {
+    for (auto const& process_name : _driver.process_names()) {
+
+      // Get the process ID by name lookup
+      ProcessID_t pid = _driver.process_id(process_name);
       auto proc_ptr = _driver.process_ptr(pid);
+
+      // auto proc_ptr = _driver.process_ptr(pid);
       if (!(proc_ptr->is("BatchFiller"))) continue;
 
-      auto const& name = _process_name_v[pid];
       BatchDataState_t batch_state = BatchDataState_t::kBatchStateUnknown;
       switch ( ((BatchHolder*)(proc_ptr))->data_type() ) {
       case BatchDataType_t::kBatchDataShort:
         ((BatchFillerTemplate<short>*)proc_ptr)->_batch_data_ptr
-          = &(BatchDataQueueFactory<short>::get_writeable().get_queue_writeable(name).get_next_writeable());
+          = &(BatchDataQueueFactory<short>::get_writeable().get_queue_writeable(process_name).get_next_writeable());
         batch_state = ((BatchFillerTemplate<short>*)proc_ptr)->_batch_data_ptr->state();
         break;
       case BatchDataType_t::kBatchDataInt:
         ((BatchFillerTemplate<int>*)proc_ptr)->_batch_data_ptr
-          = &(BatchDataQueueFactory<int>::get_writeable().get_queue_writeable(name).get_next_writeable());
+          = &(BatchDataQueueFactory<int>::get_writeable().get_queue_writeable(process_name).get_next_writeable());
         batch_state = ((BatchFillerTemplate<int>*)proc_ptr)->_batch_data_ptr->state();
         break;
       case BatchDataType_t::kBatchDataFloat:
         ((BatchFillerTemplate<float>*)proc_ptr)->_batch_data_ptr
-          = &(BatchDataQueueFactory<float>::get_writeable().get_queue_writeable(name).get_next_writeable());
+          = &(BatchDataQueueFactory<float>::get_writeable().get_queue_writeable(process_name).get_next_writeable());
         batch_state = ((BatchFillerTemplate<float>*)proc_ptr)->_batch_data_ptr->state();
         break;
       case BatchDataType_t::kBatchDataDouble:
         ((BatchFillerTemplate<double>*)proc_ptr)->_batch_data_ptr
-          = &(BatchDataQueueFactory<double>::get_writeable().get_queue_writeable(name).get_next_writeable());
+          = &(BatchDataQueueFactory<double>::get_writeable().get_queue_writeable(process_name).get_next_writeable());
         batch_state = ((BatchFillerTemplate<double>*)proc_ptr)->_batch_data_ptr->state();
         break;
+      case BatchDataType_t::kBatchDataParticle:
+        ((BatchFillerTemplate<larcv3::ParticleHolder>*)proc_ptr)->_batch_data_ptr
+          = &(BatchDataQueueFactory<larcv3::ParticleHolder>::get_writeable().get_queue_writeable(process_name).get_next_writeable());
+        batch_state = ((BatchFillerTemplate<larcv3::ParticleHolder>*)proc_ptr)->_batch_data_ptr->state();
+        break;
       default:
-        LARCV_CRITICAL() << "Process name " << name
+        LARCV_CRITICAL() << "Process process_name " << process_name
                          << " encountered none-supported BatchDataType_t: " << (int)(((BatchHolder*)(proc_ptr))->data_type()) << std::endl;
         throw larbys();
       }
@@ -489,6 +460,8 @@ int omp_thread_count() {
         ((BatchFillerTemplate<float>*)ptr)->batch_begin(); break;
       case BatchDataType_t::kBatchDataDouble:
         ((BatchFillerTemplate<double>*)ptr)->batch_begin(); break;
+      case BatchDataType_t::kBatchDataParticle:
+        ((BatchFillerTemplate<larcv3::ParticleHolder>*)ptr)->batch_begin(); break;
       default:
         LARCV_CRITICAL() << " encountered none-supported BatchDataType_t: " << (int)(((BatchHolder*)(ptr))->data_type()) << std::endl;
         throw larbys();
@@ -511,6 +484,8 @@ int omp_thread_count() {
         ((BatchFillerTemplate<float>*)ptr)->batch_end(); break;
       case BatchDataType_t::kBatchDataDouble:
         ((BatchFillerTemplate<double>*)ptr)->batch_end(); break;
+      case BatchDataType_t::kBatchDataParticle:
+        ((BatchFillerTemplate<larcv3::ParticleHolder>*)ptr)->batch_end(); break;
       default:
         LARCV_CRITICAL() << " encountered none-supported BatchDataType_t: " << (int)(((BatchHolder*)(ptr))->data_type()) << std::endl;
         throw larbys();
@@ -533,36 +508,43 @@ void init_queueprocessor(pybind11::module m){
   queueproc.def(pybind11::init<std::string>(),
     pybind11::arg("name") = "QueueProcessor");
 
-  queueproc.def("batch_process",          &Class::batch_process);
-  queueproc.def("prepare_next",     &Class::prepare_next);
-  queueproc.def("reset",     &Class::reset);
+  queueproc.def("batch_process",                    &Class::batch_process);
+  queueproc.def("prepare_next",                     &Class::prepare_next);
+  queueproc.def("reset",                            &Class::reset);
   queueproc.def("configure",
-    (void (Class::*)(const std::string, int)) (&Class::configure),
-    pybind11::arg("config_file"),
-    pybind11::arg("color")=0);
-  queueproc.def("configure",
-    (void (Class::*)(const larcv3::PSet&, int)) (&Class::configure),
+    (void (Class::*)(const json&, int)) (           &Class::configure),
     pybind11::arg("cfg"),
     pybind11::arg("color")=0);
-  queueproc.def("configured",         &Class::configured);
-  queueproc.def("pop_current_data",         &Class::pop_current_data);
-  queueproc.def("set_next_index",         &Class::set_next_index);
+  queueproc.def("configured",                       &Class::configured);
+  queueproc.def("pop_current_data",                 &Class::pop_current_data);
+  queueproc.def("set_next_index",                   &Class::set_next_index);
   queueproc.def("set_next_batch",
-    (void (Class::*)(const std::vector<size_t>&)) (&Class::set_next_batch));
+    (void (Class::*)(const std::vector<size_t>&)) ( &Class::set_next_batch));
 
   queueproc.def("set_next_batch",
     (void (Class::*)(pybind11::array_t <size_t> )) (&Class::set_next_batch));
 
-  queueproc.def("is_reading",         &Class::is_reading);
-  queueproc.def("get_n_entries",         &Class::get_n_entries);
-  queueproc.def("processed_entries",         &Class::processed_entries);
-  queueproc.def("processed_events",         &Class::processed_events);
-  queueproc.def("pd",         &Class::pd);
-  queueproc.def("storage_name",         &Class::storage_name);
-  queueproc.def("process_id",         &Class::process_id);
-  queueproc.def("batch_fillers",         &Class::batch_fillers);
-  queueproc.def("batch_types",         &Class::batch_types);
-
+  queueproc.def("is_next_ready",                    &Class::is_next_ready);
+  queueproc.def("is_reading",                       &Class::is_reading);
+  queueproc.def("get_n_entries",                    &Class::get_n_entries);
+  queueproc.def("processed_entries",                &Class::processed_entries);
+  queueproc.def("processed_events",                 &Class::processed_events);
+  queueproc.def("pd",                               &Class::pd);
+  queueproc.def("storage_name",                     &Class::storage_name);
+  queueproc.def("process_id",                       &Class::process_id);
+  queueproc.def("batch_fillers",                    &Class::batch_fillers);
+  queueproc.def("batch_types",                      &Class::batch_types);
+  queueproc.def("default_config",                   &Class::default_config);
+  std::string name = "get_queue_" + larcv3::as_string<short>();
+  queueproc.def(name.c_str(),                       &Class::get_queue<short>);
+  name = "get_queue_" + larcv3::as_string<int>();
+  queueproc.def(name.c_str(),                       &Class::get_queue<int>);
+  name = "get_queue_" + larcv3::as_string<float>();
+  queueproc.def(name.c_str(),                       &Class::get_queue<float>);
+  name = "get_queue_" + larcv3::as_string<double>();
+  queueproc.def(name.c_str(),                       &Class::get_queue<double>);
+  name = "get_queue_particle";
+  queueproc.def(name.c_str(),                       &Class::get_queue<larcv3::ParticleHolder>);
 
 }
 
