@@ -5,17 +5,15 @@
  *
  * \brief Class def header for a class QueueProcessor
  *
- * @author kazuhiro
+ * @author cadams
  */
 
-/** \addtogroup ThreadIO
-
-    @{*/
 #ifndef __LARCV3THREADIO_QUEUEPROCESSOR_H
 #define __LARCV3THREADIO_QUEUEPROCESSOR_H
 
 #include "larcv3/core/processor/ProcessDriver.h"
 #include "QueueIOTypes.h"
+#include "BatchDataQueue.h"
 #include <random>
 #include <future>
 
@@ -41,20 +39,18 @@ namespace larcv3 {
     /// Default destructor
     ~QueueProcessor();
 
-    // Process a batch of entries, using _next_index_v to specify entries
+    /// Process a batch of entries, using _next_index_v to specify entries
     bool batch_process();
 
-    // Spawn a thread to batch process and return immediately
+    /// Spawn a thread to batch process and return immediately
     void prepare_next();
 
-    // Reset the state
+    /// Reset the state
     void reset();
 
-    // configure the processor from a file on disk
-    void configure(const std::string config_file, int color=0);
 
     // configure the processor from a PSet object
-    void configure(const PSet& cfg, int color=0);
+    void configure(const json& cfg, int color=0);
 
     // Check if the processor is configured
     inline bool configured() const { return _configured;}
@@ -69,6 +65,15 @@ namespace larcv3 {
     // Set the next set of indexes to read:
     void set_next_batch(const std::vector<size_t>& index_v);
 
+    static json default_config(){
+        json c = {
+            {"Verbosity", 2},
+            {"InputFiles", std::vector<std::string>()},
+            {"ProcessName", 0},
+            {"ProcessDriver", ProcessDriver::default_config()},
+        };
+        return c;
+    }
 
     // These functions only appear in larcv proper, not in included headers:
 #ifdef LARCV_INTERNAL
@@ -79,7 +84,12 @@ namespace larcv3 {
 
 
     // Return true only if the fillers are preparing the next batch
-    bool is_reading() const {return _processing;}
+    bool is_reading() const;
+
+    bool is_next_ready() const;
+
+    template<class T>
+    const BatchDataQueue<T> & get_queue(std::string process_name) const;
 
     // Get number of entries possible to read
     size_t get_n_entries() const;
@@ -105,6 +115,8 @@ namespace larcv3 {
 
   private:
 
+    json config;
+
     bool set_batch_storage();
     bool begin_batch();
     bool end_batch();
@@ -113,7 +125,6 @@ namespace larcv3 {
     bool _configured;
     std::vector<size_t> _next_index_v;
 
-    std::vector<std::string> _input_fname_v;
     size_t _batch_global_counter;
 
     // This controls the different pieces of a batch (data, image, label, pid, etc)
