@@ -18,7 +18,6 @@
 
 namespace larcv3 {
 
-  /// Global larcv3::EventSparseTensorFactory to register EventSparseTensor
   static EventSparseTensorFactory<2> __global_EventSparseTensor2DFactory__;
   static EventSparseTensorFactory<3> __global_EventSparseTensor3DFactory__;
 
@@ -129,7 +128,7 @@ namespace larcv3 {
       H5Pset_deflate(extents_cparms, compression);
       // extents_cparms.setDeflate(compression);
     }
-    
+
     hid_t lcpl = H5Pcreate(H5P_LINK_CREATE);
     hid_t dapl = H5Pcreate(H5P_DATASET_ACCESS);
 
@@ -283,7 +282,7 @@ namespace larcv3 {
 
        _open_in_datasets[VOXELS_DATASET]          = H5Dopen(group, "voxels", H5P_DEFAULT);
        _open_in_dataspaces[VOXELS_DATASET]        = H5Dget_space(_open_in_datasets[VOXELS_DATASET]);
-;
+
      }
 
     return;
@@ -634,6 +633,10 @@ namespace larcv3 {
     }
     hid_t xfer_plist_id = H5Pcreate(H5P_DATASET_XFER);
 
+#ifdef LARCV_MPI
+    H5Pset_dxpl_mpio(xfer_plist_id, H5FD_MPIO_COLLECTIVE);
+#endif
+
     open_in_datasets(group);
     /////////////////////////////////////////////////////////
     // Step 1: Get the extents information from extents dataset
@@ -824,24 +827,25 @@ namespace larcv3 {
       // This implementation is not ideal.
       // It copies from disk, then is copying into a vector
 
-      std::vector<larcv3::Voxel> temp_voxel_vector;
-      temp_voxel_vector.resize(voxels_slab_dims[0]);
+      // std::vector<larcv3::Voxel> temp_voxel_vector;
 
       // Reserve space for reading in voxels:
 
+      // Directly get the vector object for the voxels:
+      std::vector<Voxel> & voxel_ref = _tensor_v.at(voxel_set_index).writeable_voxel_vector();
+      voxel_ref.resize(voxels_slab_dims[0]);
+
+
       H5Dread(
         _open_in_datasets[VOXELS_DATASET],    // hid_t dataset_id  IN: Identifier of the dataset read from.
         _data_types[VOXELS_DATASET],          // hid_t mem_type_id IN: Identifier of the memory datatype.
         voxels_memspace,                      // hid_t mem_space_id  IN: Identifier of the memory dataspace.
         _open_in_dataspaces[VOXELS_DATASET],  // hid_t file_space_id IN: Identifier of the dataset's dataspace in the file.
         xfer_plist_id,                        // hid_t xfer_plist_id     IN: Identifier of a transfer property list for this I/O operation.
-        &(temp_voxel_vector[0])               // void * buf  OUT: Buffer to receive data read from file.
+        &(voxel_ref[0])               // void * buf  OUT: Buffer to receive data read from file.
       );
       // std::cout << "temp_voxel_vector.size(): " << temp_voxel_vector.size() << std::endl;
 
-      for (auto & v : temp_voxel_vector){
-        _tensor_v.at(voxel_set_index).emplace(v);
-      }
 
 
       _tensor_v.at(voxel_set_index).id(voxel_set_index);
