@@ -1,7 +1,7 @@
 /**
  * \file ImageMeta.h
  *
- * \ingroup core_DataFormat
+ * \ingroup DataFormat
  *
  * \brief Class def header for a class larcv3::ImageMeta
  *
@@ -16,7 +16,7 @@
  * An "index" is a unique, single value for each coordinate.
  */
 
-/** \addtogroup core_DataFormat
+/** \addtogroup DataFormat
 
     @{*/
 #ifndef __LARCV3DATAFORMAT_IMAGEMETA_H__
@@ -165,7 +165,7 @@ class ImageMeta {
   // }
 
   inline size_t cols() const {return _number_of_voxels[0];}
-  inline size_t rows() const {return _number_of_voxels[1];}
+  size_t rows() const;
 
   // inline void update(size_t row_count, size_t col_count) {
   //   _row_count = row_count;
