@@ -42,6 +42,9 @@ void init_dataformat(pybind11::module m){
     wrap_vector<larcv3::ImageMeta<3>>(m);
     wrap_vector<larcv3::ImageMeta<4>>(m);
 
+    wrap_vector<larcv3::BBox<2>>(m);
+    wrap_vector<larcv3::BBox<3>>(m);
+
     wrap_vector<larcv3::Particle> (m);
 
     init_dataformattypes(m);
@@ -58,5 +61,6 @@ void init_dataformat(pybind11::module m){
     init_eventsparsecluster(m);
     init_eventsparsetensor(m);
     init_eventtensor(m);
+    init_eventbbox(m);
     init_iomanager(m);
 }
