@@ -1,14 +1,14 @@
 /**
  * \file larbys.h
  *
- * \ingroup core_Base
+ * \ingroup DataFormat
  *
  * \brief Class def header for exception classes for larcv3 framework
  *
  * @author cadams
  */
 
-/** \addtogroup core_Base
+/** \addtogroup DataFormat
 
     @{*/
 #ifndef __LARCV3DATAFORMAT_DATAFORMAT_H__
@@ -24,6 +24,7 @@
 #include "EventSparseCluster.h"
 #include "EventSparseTensor.h"
 #include "EventTensor.h"
+#include "EventBBox.h"
 #include "ImageMeta.h"
 #include "IOManager.h"
 #include "Particle.h"
